# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Form for curation-related user settings, inspired by the notifications settings."""

from flask_wtf import FlaskForm
from wtforms import BooleanField


class CurationPreferencesProxy:
    """Proxy class giving direct access to a user's curation preferences."""

    def __init__(self, user):
        """Constructor."""
        super().__setattr__("_user", user)

    def __getattr__(self, attr):
        """Get the attribute's value in the user's curation preferences."""
        if attr == "_user":
            return self._user

        return self._user.preferences.get("curation", {}).get(attr, None)

    def __setattr__(self, attr, value):
        """Set the attribute's value in the user's curation preferences."""
        self._user.preferences = {
            **self._user.preferences,
            "curation": {
                **self._user.preferences.get("curation", {}),
                attr: value,
            },
        }

    def __hasattr__(self, attr):
        """Check if the user's curation preferences have the given attribute."""
        return attr in self._user.preferences.get("curation", {})


class CurationForm(FlaskForm):
    """Form for editing a user's curation preferences."""

    proxy_cls = CurationPreferencesProxy

    consent = BooleanField(
        "Consent to curation of my records",
        description=(
            "Allow the repository team to curate the metadata of my records, by e.g. fixing typos and adding new related works as they are reported."  # noqa
        ),
    )

    def process(self, formdata=None, obj=None, data=None, extra_filters=None, **kwargs):
        """Build a proxy around the object."""
        if obj is not None:
            obj = self.proxy_cls(obj)

        return super().process(
            formdata=formdata, obj=obj, data=data, extra_filters=extra_filters, **kwargs
        )

    def populate_obj(self, user):
        """Populate the object."""
        user = self.proxy_cls(user)
        return super().populate_obj(user)
